use crate::{
    asset_manager::AssetDefinition,
    sqlite::{Batch, Sqlite},
    AccountId, Amount, AssetManager, Ledger, Status, TransactionId,
};
use sqlx::sqlite::SqlitePoolOptions;

pub async fn get_instance() -> (
    AssetManager,
    Ledger<'static, Batch<'static>, Sqlite<'static>>,
) {
    let pool = SqlitePoolOptions::new()
        .max_connections(1)
        .idle_timeout(None)
        .max_lifetime(None)
        .connect(":memory:")
        //.connect("sqlite:///tmp/test.db")
        .await
        .expect("pool");

    let assets = AssetManager::new(vec![
        AssetDefinition::new(1, "BTC", 8),
        AssetDefinition::new(2, "USD", 4),
    ]);

    let db = Sqlite::new(pool, assets.clone());
    db.setup().await.expect("setup");
    (assets, Ledger::new(db))
}

pub async fn deposit(
    ledger: &Ledger<'static, Batch<'static>, Sqlite<'static>>,
    account_id: &AccountId,
    amount: Amount,
) -> TransactionId {
    ledger
        .deposit(account_id, amount, Status::Settled, "Test".to_owned())
        .await
        .expect("valid tx")
        .id()
        .clone()
}

mod deposit;
mod negative_deposite;
